#!/bin/bash

# extract a five-band colour catalogue from the CARS/CFHTLS D1
# images for stellar objects; many parts of the script are copy/paste
# from the 'ldac_demo.sh'

#where are the images
IMDIR=/tmp_mnt/netpapeur/users_home2/visi09/Paris_DUEL/data

# just run SExtractor on all filters:
for FILT in u g r i z
do
  IMAGE=${IMDIR}/D1_${FILT}.V1.6A_Paris.cut.fits

  echo "sextracting from ${IMAGE}"
  MAGZP=`dfits ${IMAGE} | fitsort -d MAGZP | awk '{print $2}'`
  GAIN=`dfits ${IMAGE} | fitsort -d GAIN | awk '{print $2}'`
  SEEING=`dfits ${IMAGE} | fitsort -d SEEING | awk '{print $2}'`
  PIXSCALE=`dfits ${IMAGE} | fitsort -d CDELT1 | awk '{print -$2 * 3600}'`

  sex_theli ${IMAGE} -c extract_colour.sex\
            -CATALOG_TYPE FITS_LDAC \
            -DETECT_MINAREA 5\
            -DETECT_THRESH 3\
            -MAG_ZEROPOINT ${MAGZP} \
            -GAIN ${GAIN} \
            -SEEING_FWHM ${SEEING} \
            -PIXEL_SCALE ${PIXSCALE} \
            -CATALOG_NAME ${FILT}_sex.cat

  ldacconv -i ${FILT}_sex.cat \
           -o ${FILT}_ldac.cat \
           -b 1 -c MEGAPRIME -f test
done

associate -i i_ldac.cat z_ldac.cat r_ldac.cat g_ldac.cat u_ldac.cat\
          -o tmp1.cat tmp2.cat tmp3.cat tmp4.cat tmp5.cat\
          -c associate.conf

# take care of the 'order' of images in the associate call
# and the channels in the make_ssc.conf file:
make_ssc -i  tmp1.cat tmp2.cat tmp3.cat tmp4.cat tmp5.cat \
         -o merged.cat -c make_ssc.conf


# filter for bright stars; we do this purely on i-band CLASS_STAR
# here:
ldacfilter -i merged.cat -t PSSC -o stars.cat \
           -c "(CLASS_STAR_i>0.95)AND(mag_i<22.0);"

# create ASCII files for the colour-colour plots:
ldactoasc -b -i stars.cat -t PSSC -k mag_g mag_r mag_i |\
  awk '$1 > 0 && $2 > 0 && $3 > 0' > gri.asc
ldactoasc -b -i stars.cat -t PSSC -k mag_g mag_i mag_z |\
  awk '$1 > 0 && $2 > 0 && $3 > 0' > giz.asc
ldactoasc -b -i stars.cat -t PSSC -k mag_u mag_g mag_r |\
  awk '$1 > 0 && $2 > 0 && $3 > 0' > ugr.asc

# get corresponding files from the Pickles catalogue:
ldactoasc -i Pickles.cat -t MEGAPRIME \
  -k MAG_MEGAPRIME_g MAG_MEGAPRIME_r MAG_MEGAPRIME_i > gri_pickles.asc

ldactoasc -i Pickles.cat -t MEGAPRIME \
  -k MAG_MEGAPRIME_g MAG_MEGAPRIME_i MAG_MEGAPRIME_z > giz_pickles.asc

ldactoasc -i Pickles.cat -t MEGAPRIME \
  -k MAG_MEGAPRIME_u MAG_MEGAPRIME_g MAG_MEGAPRIME_r > ugr_pickles.asc

