#!/bin/bash

# Simple script to visualise some Skymaker PSF entrance pupils:
# This script needs installed:
# - Skymaker
# - The THELI pipeline
# - Emmanuel Bertins 'stiff' program 
#   (see http://terapix.iap.fr/rubrique.php?id_rubrique=178) 
#   to convert FITS images to TIFF files

# Just look at the outputs of this script!

# give the PATH to your skymaker executable here:
P_SKY=/home/thomas/software/marvin/skymaker-3.1.0//src/sky

# Unfortunately Skymaker currently only support one checkimage
# at a time:
# 'Simple' Mirror pupil; no secondary, no spider arms etc:
${P_SKY} -IMAGE_NAME simple_pupil.fits \
         -IMAGE_TYPE PUPIL_REAL \
         -SEEING_TYPE NO_SEEING \
         -PSF_OVERSAMP 11 \
         -ARM_COUNT 0 \
         -M2_DIAMETER 0.0

makesubimage 256 256 512 512 \
             < simple_pupil.fits > simple_pupil_small.fits

stiff simple_pupil_small.fits
convert -negate stiff.tif simple_pupil.jpg

${P_SKY} -IMAGE_NAME simple_pupil_psf.fits \
         -IMAGE_TYPE PSF_FULLRES \
         -SEEING_TYPE NO_SEEING \
         -PSF_OVERSAMP 11 \
         -ARM_COUNT 0 \
         -M1_DIAMETER 0.5 \
         -M2_DIAMETER 0.0

makesubimage 256 256 512 512 \
             < simple_pupil_psf.fits > simple_pupil_psf_small.fits

stiff simple_pupil_psf_small.fits
convert -negate stiff.tif simple_pupil_psf.jpg

${P_SKY} -IMAGE_NAME real_pupil.fits \
         -IMAGE_TYPE PUPIL_REAL \
         -SEEING_TYPE NO_SEEING \
         -PSF_OVERSAMP 11 \
         -ARM_COUNT 4 \
         -M1_DIAMETER 3.6 \
         -M2_DIAMETER 1.0

makesubimage 256 256 512 512 \
             < real_pupil.fits > real_pupil_small.fits

stiff real_pupil_small.fits
convert -negate stiff.tif real_pupil.jpg


${P_SKY} -IMAGE_NAME real_pupil_psf.fits \
         -IMAGE_TYPE PSF_FULLRES \
         -SEEING_TYPE NO_SEEING \
         -PSF_OVERSAMP 11 \
         -ARM_COUNT 4 \
         -M1_DIAMETER 0.5 \
         -M2_DIAMETER 0.15

makesubimage 256 256 512 512 \
             < real_pupil_psf.fits > real_pupil_psf_small.fits

stiff real_pupil_psf_small.fits
convert -negate stiff.tif real_pupil_psf.jpg

${P_SKY} -IMAGE_NAME astig_pupil.fits \
         -IMAGE_TYPE PUPIL_PHASE \
         -SEEING_TYPE NO_SEEING \
         -PSF_OVERSAMP 11 \
         -ARM_COUNT 4 \
         -M1_DIAMETER 3.6 \
         -M2_DIAMETER 1.0 \
         -AST00_D80 0.5 -AST45_D80 0.2

makesubimage 256 256 512 512 \
             < astig_pupil.fits > astig_pupil_small.fits

stiff astig_pupil_small.fits
convert -negate stiff.tif astig_pupil.jpg

${P_SKY} -IMAGE_NAME astig_pupil_psf.fits \
         -IMAGE_TYPE PSF_FULLRES \
         -SEEING_TYPE LONG_EXPOSURE \
         -PSF_OVERSAMP 11 \
         -ARM_COUNT 4 \
         -M1_DIAMETER 0.5 \
         -M2_DIAMETER 0.15

makesubimage 256 256 512 512 \
             < astig_pupil_psf.fits > astig_pupil_psf_small.fits

stiff astig_pupil_psf_small.fits
convert -negate stiff.tif astig_pupil_psf.jpg
