#!/bin/bash

# The script takes the five ASCII stuff catalogues generated with
# multicol_simul.sh and creates an ASCII catalogue which could
# be used with the photometric redshift code BPZ (Benitez 2000).
#
# Result is an ASCII file BPZ_prefix.asc

#$1: The prefix of your stuff catalogues (third argument from
#    multicol_simul.sh)

PREFIX=$1
MAGERR=0.05  # objects will be assigned an error in magnitude
             # of that quantity; note that from stuff we know
             # the magnitudes exactly WITHOUT error; but this
             # is NOT the real world!

# For BPZ we need to get a file with the format:
#
# ID totalmag mag_1 mag_2 .... errmag_1 errmag_2 .. z
# (totalmag is the magnitude used for the BPZ prior! Hence this
# magnitude appear twice in the produced output file! We use
# here the i-band magnitude for it. see also the manual of BPZ)
#
# The quantities mag_1, mag_2 are in the fourth columns of the
# corresponding ASCII stuff catalogues; we use 'awk' to merge
# them into one file:
awk '{magi=$4; z=$12; 
      getline<"'${PREFIX}'_z.asc"; magz=$4;
      getline<"'${PREFIX}'_r.asc"; magr=$4;
      getline<"'${PREFIX}'_g.asc"; magg=$4;
      getline<"'${PREFIX}'_u.asc"; magu=$4;
      if (magr < 24 && magu < 90 && magg < 90 && magi < 90 && magz < 90) {
      print NR, magi, magi, magz, magr, magg, 
            magu, '${MAGERR}', '${MAGERR}', 
            '${MAGERR}', '${MAGERR}', '${MAGERR}', z}}' \
      ${PREFIX}_i.asc > BPZ_${PREFIX}.asc
