#!/bin/bash

# The scripts allows you to merge an original stuff catalogue
# (converted to LDAC format) and an object catalogue extracted 
# from a corresponding skymaker image. This is for instance 
# useful to compare
# photometric with input redshifts or to evaluate completeness
# limits from SExtractor catalogues.
# The script is directly usable with the LDAC/stuff produced catalogues
# from multicol_simul.sh and associated Skymaker images (CARS D1 images).

# associate and make_ssc were originally designed to do
# object association with World coordinates (Ra, Dec).
# It is possible to do matching via pixel coordinates as well.
# Most important is that FIELDS tables certain leys (also if
# they are never used; see below). Moreover, the associate
# program MUST be called with the '-p' option.

# THE FIRST CATALOGUE NEEDS TO BE THAT EXTRACTED WITH SEXTRACTOR

#$1: first cat
#$2: second cat (can be any of the five input stuff catalogues;
#                we are mainly interested in the input redshifts)
#$3: output LDAC catalogue
#$4: ASCII catalogue for BPZ

# necessary if we match catalogues which have 'no' WCS
# but only pixel information:
ldacaddkey -i $1 -o tmp1.cat_$$ -t FIELDS \
           -k CRVAL1 0.0 DOUBLE ""\
              CRVAL2 0.0 DOUBLE ""\
              CRPIX1 0.0 DOUBLE ""\
              CRPIX2 0.0 DOUBLE ""\
              CDELT1 0.0 DOUBLE ""\
              CDELT2 0.0 DOUBLE ""

# A catalogue NOT extracted with SExtractot typically
# also does not have MAPNAXIS keywords (required by
# make_ssc)
ldacaddkey -i $2 -o tmp2.cat_$$ -t FIELDS \
           -k CRVAL1 0.0 DOUBLE ""\
              CRVAL2 0.0 DOUBLE ""\
              CRPIX1 0.0 DOUBLE ""\
              CRPIX2 0.0 DOUBLE ""\
              CDELT1 0.0 DOUBLE ""\
              CDELT2 0.0 DOUBLE ""\
              MAPNAXS1 0.0 LONG ""\
              MAPNAXS2 0.0 LONG ""

associate -i tmp1.cat_$$ tmp2.cat_$$ \
          -o tmp3.cat_$$ tmp4.cat_$$ -p -c associate_pix.conf

# filter away sources which have no match and do the match again.
# This saves us from getting rid of these objects later:
ldacfilter -i tmp3.cat_$$ -o tmp6.cat_$$ -c "(Pair_1>0);"

ldacfilter -i tmp4.cat_$$ -o tmp5.cat_$$ -c "(Pair_0>0);"

associate -i tmp6.cat_$$ tmp5.cat_$$ \
          -o tmp7.cat_$$ tmp8.cat_$$ -p -c associate_pix.conf

make_ssc -i tmp7.cat_$$ tmp8.cat_$$ -o tmp9.cat_$$ \
         -c make_ssc_pix.conf

# we want only objects with exactly one match; avoid here problems
# with blended sources etc.
ldacfilter -i tmp9.cat_$$ -o $3 -t PSSC -c "(N_00=1)AND(N_01=1);"

# write an output catalogue which can be fed into BPZ;
# watch the order of magnitudes here and in your BPZ
# columns file!
ldactoasc -b -i $3 -t PSSC -k SeqNr MAG_AUTO \
          MAG_ISO_i  MAG_ISO_z MAG_ISO_r \
          MAG_ISO_g MAG_ISO_u \
          MAGERR_ISO_i MAGERR_ISO_z \
          MAGERR_ISO_r MAGERR_ISO_g \
          MAGERR_ISO_u redshift > $4


rm tmp*cat_$$
