/*
                                  makeit.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*       Part of:        Stuff
*
*       Author:         E.BERTIN (IAP)
*
*       Contents:       Main loop
*
*       Last modify:    26/01/2005
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#ifdef HAVE_CONFIG_H
#include	"config.h"
#endif

#ifdef USE_THREADS
#include <pthread.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "define.h"
#include "globals.h"
#include "prefs.h"
#include "cosmo.h"
#include "galaxies.h"
#include "lf.h"
#include "random.h"
#include "sed.h"

/********************************** makeit ***********************************/
void	makeit(void)
  {
   galstruct		*galtype, *gal;
   sedstruct		*calibsed, *refpb, *pb,*pbt, *galsed, *tau_i, *dsed,
			*backsed, *starsed;
   lfstruct		*lf, *lfevol;
   time_t		thetime, thetime2;
   struct tm		*tm;
   FILE			*(catfile[SED_MAXNPB]), **file;
   static char		seddir_name[MAXCHAR], filterdir_name[MAXCHAR],
			extdir_name[MAXCHAR];
   char			*cptr;
   static double	xscale[SED_MAXNPB], xoffset[SED_MAXNPB],
			yscale[SED_MAXNPB], yoffset[SED_MAXNPB];
   double		mabsmax, dn, bt, kb,kd,kt, posang,cosi, btp,
			bflat,bsize,dsize,
			width,height, widthmax,heightmax, x,y, xp,yp, domega,
			z, dz,zlow,zhigh, dm, zmax, dlc, mag,mabs,mabsb,mabsd,
			da, sum1,sum2;
   int			i,g,n,p, npb, ngaltype, ngalsed, nstarsed;

/* Processing start date and time */
  thetime = time(NULL);
  tm = localtime(&thetime);
  sprintf(prefs.sdate_start,"%04d-%02d-%02d",
	tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday);
  sprintf(prefs.stime_start,"%02d:%02d:%02d",
	tm->tm_hour, tm->tm_min, tm->tm_sec);

  NFPRINTF(OUTPUT, "");
  QPRINTF(OUTPUT,
	"----- %s %s started on %s at %s with %d thread%s\n\n",
		BANNER,
		MYVERSION,
		prefs.sdate_start,
		prefs.stime_start,
		prefs.nthreads,
		prefs.nthreads>1? "s":"");

/* Set frequently used variables */
  NFPRINTF(OUTPUT, "Initializing constants...")
/* Cosmological parameters */
  H0 = prefs.h0*KM/MPC;
  H = prefs.h0/100.0;
  OmegaM = prefs.omegam;
  OmegaL = prefs.omegal;
  deltaMH = 5*log10(H);	/* Action of H0 on absolute magnitudes */

  npb = prefs.npb_name;

/* First find the largest spread in x and y */
  widthmax = heightmax = -BIG;
  for (p=0; p<npb; p++)
    {
    if ((width = prefs.pixscale[p]*prefs.width[p])>widthmax)
      widthmax = width;
    if ((height = prefs.pixscale[p]*prefs.height[p])>heightmax)
      heightmax = height;
    }
  widthmax *= 1.1;
  heightmax *= 1.1;
/* Position offsets and scales for each passband */
  for (p=0; p<npb; p++)
    {
    xscale[p] = widthmax/prefs.pixscale[p];
    yscale[p] = heightmax/prefs.pixscale[p];
    xoffset[p] = (xscale[p]-prefs.width[p])/2.0;
    yoffset[p] = (yscale[p]-prefs.height[p])/2.0;
    }
  domega = widthmax*heightmax*ARCSEC*ARCSEC;

/* Set default directories */
  sprintf(seddir_name, "%s/seds/", prefs.datadir_name);
  NFPRINTF(OUTPUT,"");
  NPRINTF(OUTPUT, "SED directory:     %s\n", seddir_name);
  sprintf(filterdir_name, "%s/filters/", prefs.datadir_name);
  NFPRINTF(OUTPUT,"");
  NPRINTF(OUTPUT, "Filter directory:  %s\n", filterdir_name);
  sprintf(extdir_name, "%s/extinct/", prefs.datadir_name);
  NFPRINTF(OUTPUT,"");
  NPRINTF(OUTPUT, "Extinct directory: %s\n", extdir_name);

/* Get the observed passbands */
  NFPRINTF(OUTPUT, "Loading passbands...")
  pb = sed_load(filterdir_name, prefs.pb_name, npb);

/* Get the reference passband */
  cptr = prefs.refpb_name;
  refpb = sed_load(filterdir_name, &cptr, 1);

/* Get the calibration SED (which defines the magnitude system) */
  NFPRINTF(OUTPUT, "Loading the calibration SED...")
  cptr = prefs.calibsed_name;
  calibsed = sed_load(seddir_name, &cptr, 1);

/* Get the sky background SED */
  if (*prefs.backsed_name != '*')
    {
    NFPRINTF(OUTPUT, "Loading the background SED...")
    cptr = prefs.backsed_name;
    backsed = sed_load(seddir_name, &cptr, 1);
    }
  else
    backsed = NULL;

/* Calibrate the passbands in the chosen magnitude system */
  NFPRINTF(OUTPUT, "Calibrating passbands...")
  pb_calib(pb, npb, refpb, calibsed, backsed);

/* Forget the calibration SED to gain memory */
  sed_end(calibsed, 1);

/* Get the galaxy SEDs */
  NFPRINTF(OUTPUT, "Loading galaxy SEDs...")
  ngalsed = prefs.ngal_sedname;
  galsed = sed_load(seddir_name, prefs.gal_sedname, ngalsed);

/* Get the star SEDs */
  NFPRINTF(OUTPUT, "Loading star SEDs...")
  nstarsed = prefs.nstar_sedname;
  starsed = sed_load(seddir_name, prefs.star_sedname, nstarsed);

/* Calibrate the SEDs */
  NFPRINTF(OUTPUT, "Calibrating galaxy SEDs...")
  for (i=0; i<ngalsed; i++)
    sed_calib(&galsed[i], refpb);

/* Get the internal extinction law (in tau units) */
  NFPRINTF(OUTPUT, "Loading the internal extinction law...")
  cptr = prefs.extinct_name;
  tau_i = sed_load(extdir_name, &cptr, 1);

/* Initialize galaxy types (luminosity functions, SEDs) */
  NFPRINTF(OUTPUT, "Initializing galaxy types...")
  ngaltype = prefs.gal_ntype;
  galtype = gal_init(galsed, ngalsed);

/*-- Normalization of extinction */
  NFPRINTF(OUTPUT, "Calibrating extinction...")
  for (i=0; i<ngaltype; i++)
    {
    QMALLOC(dsed, sedstruct, 1);
    sum1 = sed_mul(galtype[i].dsed, 1.0, refpb, 1.0, dsed);
    sum2 = sed_mul(dsed, 1.0, tau_i, 1.0, NULL);
    sed_end(dsed, 1);
    galtype[i].disk_taufact = sum2>0.0?
				-0.4*prefs.gal_iextinc[i]*sum1/sum2 : 0.0;
    }

/* Use an appropriate step (constant in comoving radial coordinate) */
  dlc = prefs.integ_zstep*MPC/H;

/* Initialize the random generator */
  NFPRINTF(OUTPUT, "Initializing galaxy random generator...")
  init_random(prefs.galseed);

/* Open the output catalogs */
  NFPRINTF(OUTPUT, "Creating catalogs...")
  for (p=0; p<npb; p++)
    if ((catfile[p] = fopen(prefs.cat_name[p],"w")) == NULL)
      error(EXIT_FAILURE,"*ERROR*: can't create ", prefs.cat_name[p]);

/* Integrate from "far away" to 0 redshift */
  NFPRINTF(OUTPUT, "Generating galaxies...")
  zmax = Z_MAX;
  for (zhigh=zmax; zhigh>0.0; zhigh=zlow)
    {
/*-- Use an appropriate step (constant in comoving radial coordinate) */
    dz = dlc/cosmo_dlc(zhigh);
    zlow = zhigh - dz;
    if (zlow<0.0)
      {
      zlow = 0.0;
      dz = zhigh - zlow;
      }
    z = (zlow+zhigh)/2.0;

  sprintf(gstr, "Generating galaxies... z =%6.2f", z);
  NFPRINTF(OUTPUT, gstr)

/*-- Loop over galaxy types */
    gal = galtype;
    for (g=ngaltype; g--; gal++)
      {
      lf = gal->lf;
      bt = gal->bt;
      if (zlow>0.0)
        {
/*------ Compute the minimum absolute magnitude of detectable galaxies */
/*------ Linear k-corrections for bulge and disk */
        kb = bt>0.0? sed_kcor(gal->bsed, refpb, zlow) : 1.0;
        kd = bt<1.0? sed_kcor(gal->dsed, refpb, zlow) : 1.0;
        if ((kt=bt*(kb-kd)+kd) < 1/BIG)
          kt = 1/BIG;
        mabsmax = prefs.maglim[1]-cosmo_dmodul(zlow)+2.5*log10(kt);
        if (lf->mabsmax < mabsmax)
          mabsmax = lf->mabsmax;
        }
      else
        mabsmax = lf->mabsmax;

/*---- Make the luminosity function evolve with redshift */
      lfevol = lf_evol(z, lf);
/*---- Compute the volume element and the local density -> dN = n*dV */
      dn = cosmo_dvol(z)*dz*domega
		*lf_intschechter(lfevol, lfevol->mabsmin, mabsmax);
      n = random_poisson(dn);
/*---- Generate each galaxy */
      for (i=n; i--;)
        {
/*------ Redshift */
        z = random_double()*(zhigh-zlow)+zlow;
/*------ Bolometric distance-modulus */
        dm = cosmo_dmodul(z);
/*------ Absolute magnitude in the reference band */
	mabs = lf_rndschechter(lfevol, lfevol->mabsmin, mabsmax);
        mabsb = mabs + gal->bdm;	  /* Bulge */
        mabsd = mabs + gal->ddm;	  /* Disk */
/*------ Shape */
        posang = 360.0*random_double() - 180.0;
        cosi = gal_cosi();
        bflat = gal_bulgeflat(cosi);
/*------ Disk SED (absorption) */
        QMALLOC(dsed, sedstruct, 1);
        sed_extinc(gal->dsed, tau_i, gal->disk_taufact*log(cosi), dsed);
/*----- Angular distance element (in arcsec/m)*/
        da = cosmo_dlum(z)*ARCSEC/((1.0+z)*(1.0+z));
        bsize = gal_bulgesize(mabsb)/da;
        dsize = gal_disksize(gal,mabsd)/da;
/*------ Position */
        x = random_double();
        y = random_double();
/*------ Now in each observed passband */
        pbt = pb;
        file = catfile;
        for (p=0; p<npb; p++, pbt++)
          {
/*------ Position in pixel coordinates*/
          xp = x*xscale[p]-xoffset[p];
          yp = y*yscale[p]-yoffset[p];
/*-------- Linear k-corrections for bulge, disk and total */
          kb = bt>0.0? sed_kcor(gal->bsed, pbt, z) : 1.0;
          kd = bt<1.0? sed_kcor(dsed, pbt, z) : 1.0;
          if ((kt=bt*(kb-kd)+kd) < 1/BIG)
            kt = 1/BIG;
/*-------- Apparent B/T in this passband */
          btp = bt*kb/kt;
/*-------- Apparent magnitude */
          mag = mabs + dm - 2.5*log10(kt);
/*          if (mag >= prefs.maglim[0]) */
            fprintf(*(file++),
		"200 %10.3f %10.3f %8.4f %5.3f "
		"%9.3f %5.3f %+7.2f %9.3f %5.3f %+7.2f %8.5f\n",
		xp,yp, mag, btp, bsize, bflat, posang, dsize, cosi, posang, z);
          }
        sed_end(dsed, 1);
        }
      }
    }

/* Close catalogs */
  NFPRINTF(OUTPUT, "Closing catalogs...")
  for (p=0; p<npb; p++)
    if (fclose(catfile[p]))
      error(EXIT_FAILURE,"*ERROR*: can't close ", prefs.cat_name[p]);

/* Free memory */
  NFPRINTF(OUTPUT, "Freeing memory...")
  gal_end(galtype, ngaltype);
  sed_end(galsed, ngalsed);
  sed_end(pb, npb);
  sed_end(refpb, 1);
  sed_end(tau_i, 1);

/* Processing end date and time */
  thetime2 = time(NULL);
  tm = localtime(&thetime2);
  sprintf(prefs.sdate_end,"%04d-%02d-%02d",
	tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday);
  sprintf(prefs.stime_end,"%02d:%02d:%02d",
	tm->tm_hour, tm->tm_min, tm->tm_sec);
  prefs.time_diff = difftime(thetime2, thetime);

  return;
  }

